import { Observable } from "rxjs";
import { mergeMap } from "rxjs/operators";
import { IRxExecutorConfig, IRxExecuteFn, IExecution, IExecutionState, ProcessingType } from ".";
export declare const getTypeOperation: (processingType: ProcessingType) => typeof mergeMap;
export declare class Execution<P, D> implements IExecution<P, D> {
    key: string;
    state: IExecutionState<D>;
    onQueueTime: Date;
    params: P;
    onProcessingTime?: Date;
    onCompleteTime?: Date;
    onCanceledTime?: Date;
    _errors: any[];
    index: number;
    constructor(key: string);
    setWaiting(params: P, index: number): this;
    setCanceled(key: string): this;
    setProcessing(): this;
    setSuccess(data: D): this;
    setError(error: any): this;
    setData(data: D): void;
    get isLoading(): boolean;
    get isError(): boolean;
    get isSuccess(): boolean;
    get isWaiting(): boolean;
    get isCanceled(): boolean;
    get data(): D | undefined;
    get error(): any;
    get errors(): any[];
    get status(): import("./model").EnumStatusType | undefined;
    addError(error: any): void;
}
export declare class RxConcurrentExecutor<P = void, D = any> {
    protected executeFn$: IRxExecuteFn<P, D>;
    execute(params: P): Execution<P, D>;
    isLoading(key: string): boolean | undefined;
    isError(key: string): boolean | undefined;
    isSuccess(key: string): boolean | undefined;
    data(key: string): D | undefined;
    error(key: string): any;
    errors(key: string): any[] | undefined;
    status(key: string): import("./model").EnumStatusType | undefined;
    retry(key: string): Execution<P, D> | undefined;
    setData(key: string, data: D | ((prev: D) => D)): void;
    getExecution(key: string): Execution<P, D>;
    private execute$;
    private internalDestroy$;
    readonly execution$: Observable<IExecution<P, D>>;
    _executions: {
        [key: string]: Execution<P, D>;
    };
    get executions(): {
        [key: string]: Execution<P, D>;
    };
    static create<P, D>(executeFn$: IRxExecuteFn<P, D>, config?: IRxExecutorConfig<P, D>): RxConcurrentExecutor<P, D>;
    protected config: IRxExecutorConfig<P, D>;
    constructor(executeFn$: IRxExecuteFn<P, D>, config?: IRxExecutorConfig<P, D>);
    init(): void;
    destroy(): void;
}
