import { Observable } from "rxjs";
import { mergeMap } from "rxjs/operators";
import { IRetryProcessing } from "../model";
declare type ExecutionStatus = "RETRYING" | "WAITING" | "PROCESSING" | "SUCCESS" | "FAILED" | "CANCELLED";
interface IExecutionState<P, D> {
    id?: string;
    params: P;
    hashedParams?: string;
    status?: ExecutionStatus;
    data?: D;
    error: any;
}
export declare class Execution2<P, D> implements IExecutionState<P, D> {
    params: P;
    config: {
        executeFn$?: IRxExecuteFn2<P, D>;
        cache?: boolean;
        getId?: (params: P) => string;
        ignoreCache?: boolean;
    };
    id: string;
    status?: ExecutionStatus;
    data?: D;
    error: any;
    hashedParams?: string;
    private _state$;
    get state$(): Observable<Execution2<P, D>>;
    constructor(params: P, config: {
        executeFn$?: IRxExecuteFn2<P, D>;
        cache?: boolean;
        getId?: (params: P) => string;
        ignoreCache?: boolean;
    });
    get state(): IExecutionState<P, D>;
    retrying(): IExecutionState<P, D>;
    wait(): IExecutionState<P, D>;
    processing(): IExecutionState<P, D>;
    succeed(data: D): IExecutionState<P, D>;
    failed(error: any): IExecutionState<P, D>;
    cancelled(): IExecutionState<P, D>;
    listen(callback: (execution: Execution2<P, D>) => void): void;
}
declare type ProcessingType = "CONCAT" | "EXHAUST" | "MERGE" | "SWITCH";
export declare const getTypeOperation2: (processingType: ProcessingType) => typeof mergeMap;
declare type OnWaitType<P> = (params: P, context?: {
    [key: string]: any;
}) => void;
declare type OnProcessingType<P> = (params: P, context?: {
    [key: string]: any;
}) => void;
declare type OnCancelType<P> = (params: P, context?: {
    [key: string]: any;
}) => void;
declare type OnSuccessType<P, D> = (data: D, params: P, context?: {
    [key: string]: any;
}) => void;
declare type OnErrorType<P> = (error: any, params: P, context?: {
    [key: string]: any;
}) => void;
export interface IExecutor<P, D> {
    enabledRetry?: boolean;
    sequential?: "SWITCH" | "EXHAUST" | "MERGE";
    concurrent?: {
        type?: "SWITCH" | "EXHAUST" | "MERGE" | "CONCAT";
        detailed?: boolean;
        mergeCapacity: number;
    };
    params$?: Observable<P>;
    wrapData$?: (data$: Observable<D>) => Observable<D>;
    cache?: boolean;
    cacheLifetimeime?: number;
    context?: {
        [key: string]: any;
    };
    onRetry?: (error: any, params: P) => void;
    onWait?: OnWaitType<P>;
    onProcessing?: OnProcessingType<P>;
    onCancel?: OnCancelType<P>;
    onSuccess?: OnSuccessType<P, D>;
    onError?: OnErrorType<P>;
    getId?: (params: P) => string;
    retry?: IRetryProcessing;
}
export declare type IRxExecuteFn2<P, D> = (params: P) => D | Promise<D> | Observable<D>;
export declare class RxExecutor2<P, D> {
    private executeFn$;
    private config?;
    private _status?;
    private _data?;
    private _params?;
    private _error;
    get params(): P | undefined;
    get error(): any;
    get status(): ExecutionStatus | undefined;
    get data(): D | undefined;
    get isLoading(): boolean;
    get isProcessing(): boolean;
    get isSuccess(): boolean;
    get isError(): boolean;
    retry(): Execution2<P, D>;
    private _internalExecution$;
    private executionType;
    private operationType;
    private processorCapacity;
    private enableCache;
    private concurrentDetailed;
    constructor(executeFn$: IRxExecuteFn2<P, D>, config?: IExecutor<P, D> | undefined);
    get state$(): Observable<IExecutionState<P, D>>;
    _state$: Observable<IExecutionState<P, D>>;
    close(): void;
    private cachedData;
    private cachedExecution;
    static create<P, D>(executeFn$: IRxExecuteFn2<P, D>, config?: IExecutor<P, D>): RxExecutor2<P, D>;
    _execution?: Execution2<P, D>;
    getExecution(id?: string): Execution2<P, D> | undefined;
    execute(params: P, config?: {
        executeFn$: IRxExecuteFn2<P, D>;
        getId?: (params: P) => string;
        context?: {
            [key: string]: any;
        };
        onWait?: OnWaitType<P>;
        onRetry: (error: any, params: P) => void;
        onProcessing?: OnProcessingType<P>;
        onCancel?: OnCancelType<P>;
        onSuccess?: OnSuccessType<P, D>;
        onError?: OnErrorType<P>;
        ignoreCache?: boolean;
    }): Execution2<P, D>;
    private _execute;
    private processingExecutions;
    private failedExecutions;
    invalidateCache(params: any): boolean;
    clearCache(): void;
    isFull(): boolean;
    getState$(): Observable<IExecutionState<P, D>>;
}
export {};
