"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

exports.__esModule = true;
exports.getTypeOperation = exports.RxConcurrentExecutor = exports.Execution = void 0;

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _rxjs = require("rxjs");

var _operators = require("rxjs/operators");

var _lang = require("./utils/lang");

const generateUUID = () => {
  var d = new Date().getTime();
  var d2 = typeof performance !== "undefined" && performance.now && performance.now() * 1000 || 0;
  return "xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx".replace(/[xy]/g, function (c) {
    var r = Math.random() * 16;

    if (d > 0) {
      r = (d + r) % 16 | 0;
      d = Math.floor(d / 16);
    } else {
      r = (d2 + r) % 16 | 0;
      d2 = Math.floor(d2 / 16);
    }

    return (c === "x" ? r : r & 0x3 | 0x8).toString(16);
  });
};

const getTypeOperation = processingType => {
  switch (processingType) {
    case "CONCAT":
      return _operators.concatMap;

    case "EXHAUST":
      return _operators.exhaustMap;

    case "MERGE":
      return _operators.mergeMap;

    case "SWITCH":
      return _operators.switchMap;

    default:
      return _operators.switchMap;
  }
};

exports.getTypeOperation = getTypeOperation;

class Execution {
  constructor(key) {
    this.key = key;
    this.state = {};
    this.onQueueTime = void 0;
    this.params = void 0;
    this.onProcessingTime = void 0;
    this.onCompleteTime = void 0;
    this.onCanceledTime = void 0;
    this._errors = [];
    this.index = -1;
    this.setWaiting = this.setWaiting.bind(this);
    this.setCanceled = this.setCanceled.bind(this);
    this.setProcessing = this.setProcessing.bind(this);
    this.setSuccess = this.setSuccess.bind(this);
    this.setError = this.setError.bind(this);
    this.setData = this.setData.bind(this);
    this.addError = this.addError.bind(this);
  }

  setWaiting(params, index) {
    this.params = params;
    this.state = {
      status: "WAITING"
    };
    this.onQueueTime = new Date();
    return this;
  }

  setCanceled(key) {
    this.key = key;
    this.state = {
      status: "CANCELED"
    };
    this.onCanceledTime = new Date();
    return this;
  }

  setProcessing() {
    this.onProcessingTime = new Date();
    this.state = (0, _extends2.default)({}, this.state, {
      status: "PROCESSING"
    });
    return this;
  }

  setSuccess(data) {
    this.onCompleteTime = new Date();
    this.state = (0, _extends2.default)({}, this.state, {
      data,
      status: "SUCCESS"
    });
    return this;
  }

  setError(error) {
    this.onCompleteTime = new Date();
    this.state = (0, _extends2.default)({}, this.state, {
      error,
      status: "ERROR"
    });
    return this;
  }

  setData(data) {
    this.state = (0, _extends2.default)({}, this.state, {
      data
    });
  }

  get isLoading() {
    return this.state.status === "PROCESSING";
  }

  get isError() {
    return this.state.status === "ERROR";
  }

  get isSuccess() {
    return this.state.status === "SUCCESS";
  }

  get isWaiting() {
    return this.state.status === "WAITING";
  }

  get isCanceled() {
    return this.state.status === "CANCELED";
  }

  get data() {
    return this.state.data;
  }

  get error() {
    return this.state.error;
  }

  get errors() {
    return this._errors;
  }

  get status() {
    return this.state.status;
  }

  addError(error) {
    this.errors.unshift(error);
  }

}

exports.Execution = Execution;

class RxConcurrentExecutor {
  execute(params) {
    const getKey = this.config.getKey;
    const key = getKey ? getKey(params) : generateUUID();
    const execution = new Execution(key); //TODO: asyncLock

    this.executions[key] = execution;
    this.execute$.next({
      key,
      params: params
    });
    return execution;
  }

  isLoading(key) {
    const execution = this._executions[key];

    if (!execution) {
      return;
    }

    return execution.isLoading;
  }

  isError(key) {
    const execution = this._executions[key];

    if (!execution) {
      return;
    }

    return execution.isError;
  }

  isSuccess(key) {
    const execution = this._executions[key];

    if (!execution) {
      return;
    }

    return execution.isSuccess;
  }

  data(key) {
    const execution = this._executions[key];

    if (!execution) {
      return;
    }

    return execution.state.data;
  }

  error(key) {
    const execution = this._executions[key];

    if (!execution) {
      return;
    }

    return execution.state.error;
  }

  errors(key) {
    const execution = this._executions[key];

    if (!execution) {
      return;
    }

    return execution.errors;
  }

  status(key) {
    const execution = this._executions[key];

    if (!execution) {
      return;
    }

    return execution.state.status;
  }

  retry(key) {
    const execution = this._executions[key];

    if (!execution) {
      return;
    }

    return this.execute(execution.params);
  }

  setData(key, data) {
    const execution = this._executions[key];

    if (!execution) {
      return;
    }

    let newData = data;

    if (typeof data === "function") {
      newData = data(execution.state.data);
    }

    execution.setData(newData);
    this.config.onExecutionChange && this.config.onExecutionChange(execution);
  }

  getExecution(key) {
    return this._executions[key];
  }

  get executions() {
    return this._executions;
  }

  static create(executeFn$, config) {
    return new RxConcurrentExecutor(executeFn$, config);
  }

  constructor(executeFn$, config) {
    this.executeFn$ = executeFn$;
    this.execute$ = new _rxjs.Subject();
    this.internalDestroy$ = new _rxjs.Subject();
    this.execution$ = void 0;
    this._executions = {};
    this.config = void 0;
    this.config = config || {};
    this.isLoading = this.isLoading.bind(this);
    this.execute = this.execute.bind(this);
    this.retry = this.retry.bind(this);
    this.isError = this.isError.bind(this);
    this.isSuccess = this.isSuccess.bind(this);
    this.data = this.data.bind(this);
    this.error = this.error.bind(this);
    this.errors = this.errors.bind(this);
    this.status = this.status.bind(this);
    this.setData = this.setData.bind(this);
    this.getExecution = this.getExecution.bind(this);
    const effectiveDestroy$ = this.config.destroy$ ? (0, _rxjs.merge)(this.config.destroy$, this.internalDestroy$) : this.internalDestroy$;
    const execute$ = this.config.source$ ? (0, _rxjs.merge)(this.config.source$.pipe((0, _operators.map)(params => {
      const getKey = this.config.getKey;
      const key = getKey ? getKey(params) : generateUUID();
      return {
        key,
        params
      };
    }), (0, _operators.tap)(({
      key,
      params
    }) => {
      let execution = this._executions[key]; //TODO: asyncLock

      execution = new Execution(key);
      this._executions[key] = execution;
    })), this.execute$) : this.execute$;
    const asyncOperation = getTypeOperation(this.config.processingType);
    this.execution$ = execute$.pipe((0, _operators.takeUntil)(effectiveDestroy$), (0, _operators.map)((execute, index) => {
      let execution = this._executions[execute.key];
      execution.setWaiting(execute.params, index);

      if (!!Object.keys(this.executions).map(key => this.executions[key]).find(execution => execution.state.status === "PROCESSING") && this.config.processingType === "EXHAUST") {
        execution.setCanceled(execute.key);
      }

      if (this.config.processingType === "SWITCH") {
        Object.keys(this.executions).map(key => this.executions[key]).filter(execution => execution.state.status === "PROCESSING").forEach(execution => {
          execution.setCanceled(execute.key);
        });
      }

      this.config.onExecutionChange && this.config.onExecutionChange(execution);
      return execution;
    }), (0, _operators.tap)(execution => {
      this.config.onQueuing && this.config.onQueuing(execution.params, execution.key, execution);
      this.config.onExecutionChange && this.config.onExecutionChange(execution);
    }), asyncOperation(execution => {
      const params = execution.params;
      const data$ = this.executeFn$ ? this.executeFn$(params) : (0, _rxjs.of)(params).pipe((0, _operators.map)(p => p));
      const loadData$ = _lang.ModuleUtils.isObservable(data$) ? data$ : _lang.ModuleUtils.isPromise(data$) ? (0, _rxjs.from)(data$) : (0, _rxjs.of)(data$);
      execution.setProcessing();
      this._executions[execution.key] = execution;
      const retryConfig = this.config.retry ? this.config.retry : {
        interval: 1000,
        maxRetryAttempts: 0
      };
      const maxRetryAttempts = retryConfig.maxRetryAttempts ? retryConfig.maxRetryAttempts : 0;
      const notRetryWhenStatus = retryConfig.notRetryWhenStatus ? retryConfig.notRetryWhenStatus : [];
      return (0, _rxjs.merge)((0, _rxjs.of)(execution), loadData$.pipe((0, _operators.retryWhen)(errors => errors.pipe((0, _operators.tap)(e => {
        execution.addError(e);
      }), (0, _operators.concatMap)((e, i) => (0, _rxjs.iif)(() => {
        if (i < maxRetryAttempts) {
          const responseStatus = (e || {}).status || 0;
          const notRetry = !!notRetryWhenStatus.find(status => status === responseStatus);

          if (notRetry) {
            return true;
          }

          if (retryConfig.noRetryWhen && retryConfig.noRetryWhen(e)) {
            return true;
          }

          return false;
        }

        return true;
      }, (0, _rxjs.throwError)(() => e), (0, _rxjs.of)(e).pipe((0, _operators.delay)((retryConfig.typeInterval === "LINEAR" ? 1 : i) * retryConfig.interval)))))), (0, _operators.map)(data => execution.setSuccess(data))).pipe((0, _operators.catchError)(error => (0, _rxjs.of)(execution.setError(error)))));
    })).pipe((0, _operators.tap)(execution => {
      execution.state.status === "WAITING" && this.config.onQueuing && this.config.onQueuing(execution.params, execution.key, execution);
      execution.state.status === "PROCESSING" && this.config.onProcessing && this.config.onProcessing(execution.params, execution.key, execution);
      execution.state.status === "SUCCESS" && this.config.onSuccess && this.config.onSuccess(execution.state.data, execution.params, execution.key, execution);
      execution.state.status === "ERROR" && this.config.onError && this.config.onError(execution.state.error, execution.params, execution.key, execution);
    }), (0, _operators.delay)(1), (0, _operators.tap)(execution => {
      this.config.onExecutionChange && this.config.onExecutionChange(execution);
    }));
  }

  init() {
    this.execution$.subscribe();
  }

  destroy() {
    this.internalDestroy$.next();
    this.internalDestroy$.complete();
  }

}

exports.RxConcurrentExecutor = RxConcurrentExecutor;