"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

exports.__esModule = true;
exports.getTypeOperation2 = exports.RxExecutor2 = exports.Execution2 = void 0;

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _objectHash = _interopRequireDefault(require("object-hash"));

var _rxjs = require("rxjs");

var _operators = require("rxjs/operators");

var _exports = require("../exports");

const generateUniqSerial = () => {
  return "xxxx-xxxx-xxx-xxxx".replace(/[x]/g, function (c) {
    var r = Math.random() * 16 | 0,
        v = c === "x" ? r : r & 0x3 | 0x8;
    return v.toString(16);
  });
};

class Execution2 {
  get state$() {
    const state$ = this._state$.asObservable().pipe((0, _operators.shareReplay)());

    return state$;
  }

  constructor(params, config) {
    this.params = params;
    this.config = config;
    this.id = void 0;
    this.status = void 0;
    this.data = void 0;
    this.error = void 0;
    this.hashedParams = void 0;
    this._state$ = new _rxjs.Subject();
    const conf = config || {
      cache: false,
      getId: () => generateUniqSerial()
    };
    this.hashedParams = conf.cache ? (0, _objectHash.default)(params) : undefined;
    this.id = conf.getId ? conf.getId(params) : generateUniqSerial();
  }

  get state() {
    return this;
  }

  retrying() {
    this.status = "RETRYING";

    this._state$.next(this);

    this.status = "PROCESSING";

    this._state$.next(this);

    return this.state;
  }

  wait() {
    this.status = "WAITING";

    this._state$.next(this);

    return this.state;
  }

  processing() {
    this.status = "PROCESSING";

    this._state$.next(this);

    return this.state;
  }

  succeed(data) {
    this.data = data;
    this.status = "SUCCESS";

    this._state$.next(this);

    return this.state;
  }

  failed(error) {
    this.error = error;
    this.status = "FAILED";

    this._state$.next(this);

    return this.state;
  }

  cancelled() {
    this.status = "CANCELLED";

    this._state$.next(this);

    return this.state;
  }

  listen(callback) {
    this.state$.subscribe(callback);
  }

}

exports.Execution2 = Execution2;

const getTypeOperation2 = processingType => {
  switch (processingType) {
    case "CONCAT":
      return _operators.concatMap;

    case "EXHAUST":
      return _operators.exhaustMap;

    case "MERGE":
      return _operators.mergeMap;

    case "SWITCH":
      return _operators.switchMap;

    default:
      return _operators.switchMap;
  }
};

exports.getTypeOperation2 = getTypeOperation2;

class RxExecutor2 {
  get params() {
    return this._params;
  }

  get error() {
    return this._error;
  }

  get status() {
    return this._status;
  }

  get data() {
    return this._data;
  }

  get isLoading() {
    return this._status === "PROCESSING";
  }

  get isProcessing() {
    return this._status === "PROCESSING";
  }

  get isSuccess() {
    return this._status === "SUCCESS";
  }

  get isError() {
    return this._status === "FAILED";
  }

  retry() {
    return this.execute(this._params);
  }

  constructor(executeFn$, config) {
    var _tempConfig$concurren;

    this.executeFn$ = executeFn$;
    this.config = config;
    this._status = void 0;
    this._data = void 0;
    this._params = void 0;
    this._error = void 0;
    this._internalExecution$ = new _rxjs.Subject();
    this.executionType = void 0;
    this.operationType = void 0;
    this.processorCapacity = 1;
    this.enableCache = false;
    this.concurrentDetailed = false;
    this._state$ = void 0;
    this.cachedData = {};
    this.cachedExecution = {};
    this._execution = void 0;
    this.processingExecutions = [];
    this.failedExecutions = [];
    this.executionType = config ? config.concurrent ? "CONCURRENT" : "SEQUENTIAL" : "SEQUENTIAL";
    const tempConfig = this.config || {
      sequential: "SWITCH",
      concurrent: {
        type: "SWITCH",
        mergeCapacity: 1,
        detailed: true
      },
      cache: false
    };
    this.concurrentDetailed = !!((_tempConfig$concurren = tempConfig.concurrent) != null && _tempConfig$concurren.detailed);
    const {
      concurrent,
      sequential
    } = tempConfig;
    this.operationType = sequential || concurrent && concurrent.type || "SWITCH";

    if (this.operationType === "MERGE" && tempConfig.concurrent.mergeCapacity > 1) {
      this.processorCapacity = tempConfig.concurrent.mergeCapacity;
    }

    this.enableCache = !!tempConfig.cache;
    this.execute = this.execute.bind(this);
    this.close = this.close.bind(this);
    this.getExecution = this.getExecution.bind(this);
    this._state$ = this.getState$();
    this.config = config || {};
  }

  get state$() {
    return this._state$;
  }

  close() {}

  static create(executeFn$, config) {
    return new RxExecutor2(executeFn$, config);
  }

  getExecution(id) {
    return this._execution;
  }

  execute(params, config) {
    const execution = this._execute(params, config);

    this._execution = execution;

    this._internalExecution$.next(execution);

    return execution;
  }

  _execute(params, config) {
    var _this$config, _this$config2;

    const context = (config ? config.context : {}) || {};
    const mergedContext = (0, _extends2.default)({}, this.config && this.config.context || {}, context);

    if (this.executionType === "SEQUENTIAL" && this.operationType === "EXHAUST") {
      const currentxecution = this.processingExecutions[0];

      if (currentxecution) {
        return currentxecution;
      }
    }

    const execution = new Execution2(params, {
      executeFn$: config == null ? void 0 : config.executeFn$,
      cache: (_this$config = this.config) == null ? void 0 : _this$config.cache,
      getId: (_this$config2 = this.config) == null ? void 0 : _this$config2.getId,
      ignoreCache: config == null ? void 0 : config.ignoreCache
    });
    const ignoreCache = (config == null ? void 0 : config.ignoreCache) || false;
    const enableCache = this.enableCache && !ignoreCache;

    if (enableCache) {
      const key = (0, _objectHash.default)(params);
      this.cachedExecution[key] = execution;
    }

    execution.state$.subscribe(exec => {
      if (this.executionType === "SEQUENTIAL") {
        if (this.operationType === "EXHAUST" || this.operationType === "SWITCH") {
          this._params = exec.params;
          this._status = exec.status;
          if (exec.status === "SUCCESS") this._data = exec.data;
          if (exec.status === "FAILED") this._error = exec.error;
        }
      }

      if (exec.status === "RETRYING") {
        config && config.onRetry && config.onRetry(exec.error, exec.params);
        this.config && this.config.onRetry && this.config.onRetry(exec.error, exec.params);
      }

      if (exec.status === "PROCESSING") {
        config && config.onProcessing && config.onProcessing(exec.params, mergedContext);
        this.config && this.config.onProcessing && this.config.onProcessing(exec.params, mergedContext);
      }

      if (exec.status === "SUCCESS") {
        config && config.onSuccess && config.onSuccess(exec.data, exec.params, mergedContext);
        this.config && this.config.onSuccess && this.config.onSuccess(exec.data, exec.params, mergedContext);
      }

      if (exec.status === "FAILED") {
        config && config.onError && config.onError(exec.error, exec.params, mergedContext);
        this.config && this.config.onError && this.config.onError(exec.error, exec.params, mergedContext);
      }

      if (exec.status === "WAITING") {
        config && config.onWait && config.onWait(exec.params, mergedContext);
        this.config && this.config.onWait && this.config.onWait(exec.params, mergedContext);
      }

      if (exec.status === "CANCELLED") {
        config && config.onCancel && config.onCancel(exec.params, mergedContext);
        this.config && this.config.onCancel && this.config.onCancel(exec.params, mergedContext);
      }
    });
    return execution;
  }

  invalidateCache(params) {
    const key = (0, _objectHash.default)(params);
    const found = !!this.cachedData[key];
    delete this.cachedData[key];
    return found;
  }

  clearCache() {
    this.cachedData = {};
  }

  isFull() {
    return this.processingExecutions.length >= this.processorCapacity;
  }

  getState$() {
    const mergedExecutions$ = this.config && this.config.params$ ? (0, _rxjs.merge)(this._internalExecution$, this.config.params$.pipe((0, _operators.map)(params => this._execute(params)))) : this._internalExecution$;
    const asyncOperation = getTypeOperation2(this.operationType);
    const waitingSubject = new _rxjs.Subject();
    const retrySubject = new _rxjs.Subject();
    const retryConfig = this.config && this.config.retry ? this.config.retry : {
      interval: 1000,
      maxRetryAttempts: 0
    };
    const maxRetryAttempts = retryConfig.maxRetryAttempts ? retryConfig.maxRetryAttempts : 0;
    const notRetryWhenStatus = retryConfig.notRetryWhenStatus ? retryConfig.notRetryWhenStatus : [];
    return (0, _rxjs.merge)(retrySubject.pipe((0, _operators.map)(execution => {
      execution.retrying();
      return execution.state;
    })), waitingSubject.pipe((0, _operators.filter)(() => {
      return this.operationType === "SWITCH" && this.executionType === "CONCURRENT";
    }), (0, _operators.map)(execution => {
      execution.wait();
      return execution.state;
    })), mergedExecutions$.pipe((0, _operators.tap)(execution => {
      waitingSubject.next(execution);
    }), asyncOperation(execution => {
      var _execution$config, _execution$config2, _this$config3;

      const {
        params
      } = execution;
      const ignoreCache = ((_execution$config = execution.config) == null ? void 0 : _execution$config.ignoreCache) || false;
      const enableCache = this.enableCache && !ignoreCache;
      let cachedData = undefined;

      if (enableCache) {
        const key = (0, _objectHash.default)(params);
        cachedData = this.cachedData[key];
      }

      const executeFn$ = ((_execution$config2 = execution.config) == null ? void 0 : _execution$config2.executeFn$) || this.executeFn$;
      const data$ = executeFn$ ? executeFn$(params) : (0, _rxjs.of)(params).pipe((0, _operators.map)(p => p));
      const loadData$ = enableCache && cachedData !== undefined ? (0, _rxjs.of)(cachedData).pipe((0, _operators.delay)(1)) : _exports.ModuleUtils.isObservable(data$) ? data$ : _exports.ModuleUtils.isPromise(data$) ? (0, _rxjs.from)(data$) : (0, _rxjs.of)(data$);
      const wrapData$ = (_this$config3 = this.config) != null && _this$config3.wrapData$ ? this.config.wrapData$(loadData$) : loadData$;
      return (0, _rxjs.merge)((0, _rxjs.of)(execution).pipe((0, _operators.filter)(() => {
        return this.operationType === "SWITCH" && this.executionType === "CONCURRENT" && this.isFull();
      }), (0, _operators.map)(() => {
        const execution = this.processingExecutions[0];
        execution.cancelled();
        return execution.state;
      })), (0, _rxjs.of)(execution).pipe((0, _operators.filter)(() => {
        return this.operationType !== "SWITCH" || this.executionType !== "SEQUENTIAL" || !this.isFull();
      }), (0, _operators.tap)(execution => {
        this.processingExecutions.push(execution);
      }), (0, _operators.map)(execution => {
        execution.processing();
        return execution.state;
      })), wrapData$.pipe(
      /*********************************************************/
      (0, _operators.retryWhen)(errors => errors.pipe((0, _operators.tap)(e => {// execution.addError(e);
      }), (0, _operators.concatMap)((e, i) => (0, _rxjs.iif)(() => {
        if (i < maxRetryAttempts) {
          const responseStatus = (e || {}).status || 0;
          const notRetry = !!notRetryWhenStatus.find(status => status === responseStatus);

          if (notRetry) {
            return true;
          }

          if (retryConfig.noRetryWhen && retryConfig.noRetryWhen(e)) {
            return true;
          }

          return false;
        }

        return true;
      }, (0, _rxjs.throwError)(() => e), (0, _rxjs.of)(e).pipe((0, _operators.tap)(() => {
        retrySubject.next(execution);
      }), (0, _operators.delay)((retryConfig.typeInterval === "LINEAR" ? 1 : i) * retryConfig.interval)))))),
      /*********************************************************/
      (0, _operators.map)(resp => {
        execution.succeed(resp);
        return execution.state;
      }), (0, _operators.tap)(execution => {
        this.processingExecutions = this.processingExecutions.filter(e => e !== execution);

        if (this.enableCache) {
          this.cachedData[execution.hashedParams] = execution.data;
        }
      }, () => {
        this.processingExecutions = this.processingExecutions.filter(e => e !== execution);
      }), (0, _operators.catchError)(error => {
        execution.failed(error);
        return (0, _rxjs.of)(execution.state);
      })));
    }))).pipe((0, _operators.shareReplay)());
  }

}

exports.RxExecutor2 = RxExecutor2;